/*
 * Copyright (c) 2015, Jean-Baptiste Giraudeau <jb@giraudeau.info>
 *
 * This file is part of "Derive4J - Processor API".
 *
 * "Derive4J - Processor API" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * "Derive4J - Processor API" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with "Derive4J - Processor API".  If not, see <http://www.gnu.org/licenses/>.
 */
package org.derive4j.processor.api.model;

import java.util.List;

public abstract class AlgebraicDataType {

  private AlgebraicDataType() {
  }

  public static AlgebraicDataType adt(TypeConstructor typeConstructor, MatchMethod matchMethod, DataConstruction dataConstruction, List<DataArgument> fields) {
    return new AlgebraicDataType() {
      @Override
      public <R> R match(Case<R> algebraicDataType) {
        return algebraicDataType.algebraicDataType(typeConstructor, matchMethod, dataConstruction, fields);
      }
    };
  }

  public abstract <R> R match(Case<R> algebraicDataType);

  public TypeConstructor typeConstructor() {
    return match((typeConstructor, matchMethod, dataConstruction, fields) -> typeConstructor);
  }

  public MatchMethod matchMethod() {
    return match((typeConstructor, matchMethod, dataConstruction, fields) -> matchMethod);
  }

  public DataConstruction dataConstruction() {
    return match((typeConstructor, matchMethod, dataConstruction, fields) -> dataConstruction);
  }

  public List<DataArgument> fields() {
    return match((typeConstructor, matchMethod, dataConstruction, fields) -> fields);
  }

  public interface Case<R> {
    R algebraicDataType(TypeConstructor typeConstructor, MatchMethod matchMethod, DataConstruction dataConstruction, List<DataArgument> fields);
  }

}
